@@ -13,20 +13,21 @@ b <- c(0.8, 2, 1)
 df$y <- rbinom(n, 1, prob=1/(1 + exp(-(b[1] + b[2]*df$x1 + b[3]*df$x2))))
 
 test_that("logistic regression works", {
-  sampler <- create_sampler(y ~ reg(~ x1+x2, Q0=0.1), family="binomial", data=df)
+  expect_warning(sampler <- create_sampler(y ~ reg(~ x1+x2, Q0=0.1), family="binomial", data=df), "deprecated")
+  expect_equal(sampler$mod[[1L]]$prior$precision, Diagonal(x=rep.int(0.1, 3)))
   sim <- MCMCsim(sampler, n.iter=600, burnin=250, n.chain=2, verbose=FALSE)
   summ <- summary(sim)
-  expect_true(all.equal(unname(summ$reg1[, "Mean"]), b, tolerance=1))
+  expect_equal(unname(summ$reg1[, "Mean"]), b, tolerance=1)
   compute_DIC(sim)
   compute_WAIC(sim, show.progress=FALSE)
 })
 
 df$y <- rbinom(n, 1, prob=pnorm(b[1] + b[2]*df$x1 + b[3]*df$x2))
 test_that("probit regression works", {
-  sampler <- create_sampler(y ~ reg(~  x1 + x2), ny=1, family=f_binomial(link="probit"), data=df)
+  sampler <- create_sampler(y ~ reg(~ x1 + x2), ny=1, family=f_binomial(link="probit"), data=df)
   sim <- MCMCsim(sampler, n.iter=600, burnin=250, n.chain=2, verbose=FALSE)
   summ <- summary(sim)
-  expect_true(all.equal(unname(summ$reg1[, "Mean"]), b, tolerance=1))
+  expect_equal(unname(summ$reg1[, "Mean"]), b, tolerance=1)
   compute_DIC(sim)
   compute_WAIC(sim, show.progress=FALSE)
 })
@@ -63,14 +64,15 @@ test_that("minimal probit binary regression example works", {
 
 test_that("binomial multilevel model runs", {
   ex <- mcmcsae_example(1000, family="binomial")
-  set_opts(max.size.cps.template = 0)
-  sampler <- create_sampler(ex$model, data=ex$dat, family="binomial", block=TRUE)
+  sampler <- create_sampler(ex$model, data=ex$dat, family="binomial", block=TRUE,
+    control=sampler_control(max.size.cps.template = 0))
   expect_false(is.function(sampler$mbs[[1]]$cps_template))
-  set_opts(max.size.cps.template = 100)
-  sampler <- create_sampler(ex$model, data=ex$dat, family="binomial", block=TRUE)
+  sampler <- create_sampler(ex$model, data=ex$dat, family="binomial", block=TRUE,
+    control=sampler_control(max.size.cps.template=100))
   expect_true(is.function(sampler$mbs[[1]]$cps_template))
   sim <- MCMCsim(sampler, burnin=50, n.iter=100, n.chain=2, verbose=FALSE)
   summary(sim)
   sampler <- create_sampler(ex$model, data=ex$dat, family="binomial",
     block=list(c("v", "u")))
 })
+
